package mx.messaging
{
   import flash.events.Event;
   import flash.events.EventDispatcher;
   import mx.core.IMXMLObject;
   import mx.core.mx_internal;
   import mx.events.PropertyChangeEvent;
   import mx.logging.ILogger;
   import mx.logging.Log;
   import mx.messaging.config.ServerConfig;
   import mx.messaging.errors.InvalidDestinationError;
   import mx.messaging.events.ChannelEvent;
   import mx.messaging.events.ChannelFaultEvent;
   import mx.messaging.events.MessageAckEvent;
   import mx.messaging.events.MessageFaultEvent;
   import mx.messaging.messages.AbstractMessage;
   import mx.messaging.messages.AcknowledgeMessage;
   import mx.messaging.messages.AsyncMessage;
   import mx.messaging.messages.ErrorMessage;
   import mx.messaging.messages.IMessage;
   import mx.utils.Base64Encoder;
   import mx.utils.Translator;
   import mx.utils.UIDUtil;
   
   use namespace mx_internal;
   
   public class MessageAgent extends EventDispatcher implements IMXMLObject
   {
      private static const TRANSLATOR:Translator = Translator.getDefaultInstanceFor(MessageAgent);
      
      protected var _log:ILogger;
      
      private var _connected:Boolean = false;
      
      private var _clientId:String;
      
      private var _sendRemoteCredentials:Boolean;
      
      protected var _credentials:String;
      
      private var _id:String;
      
      private var _subtopic:String = "";
      
      private var _destination:String = "";
      
      private var _remoteCredentials:String = "";
      
      private var _configured:Boolean;
      
      protected var _agentType:String;
      
      private var _session:Session;
      
      private var _requestTimeout:int = -1;
      
      private var _channelSet:ChannelSet;
      
      public function MessageAgent()
      {
         _remoteCredentials = "";
         _connected = false;
         _destination = "";
         _id = UIDUtil.createUID();
         _requestTimeout = -1;
         _subtopic = "";
         super();
         _configured = true;
      }
      
      [Bindable(event="propertyChange")]
      public function get connected() : Boolean
      {
         return _connected;
      }
      
      [Bindable(event="destinationChanged")]
      public function get destination() : String
      {
         return _destination;
      }
      
      public function get session() : Session
      {
         return _session;
      }
      
      public function logout() : void
      {
         _credentials = null;
         if(channelSet)
         {
            channelSet.logout(this);
         }
      }
      
      public function get id() : String
      {
         return _id;
      }
      
      public function set destination(param1:String) : void
      {
         if(param1 == null || param1.length == 0)
         {
            throw new InvalidDestinationError(TRANSLATOR.textOf("emptyDestinationName",param1));
         }
         if(_destination != param1)
         {
            if(_configured && channelSet != null)
            {
               channelSet.disconnect(this);
               channelSet = null;
            }
            _destination = param1;
            dispatchEvent(new Event("destinationChanged"));
            if(Log.isInfo())
            {
               _log.info("\'{0}\' {2} set destination to \'{1}\'.",id,_destination,_agentType);
            }
         }
      }
      
      public function fault(param1:ErrorMessage, param2:IMessage) : void
      {
         if(Log.isError())
         {
            _log.error("\'{0}\' {2} fault for \'{1}\'.",id,param2.messageId,_agentType);
         }
         if(param1.headers[ErrorMessage.RETRYABLE_HINT_HEADER])
         {
            delete param1.headers[ErrorMessage.RETRYABLE_HINT_HEADER];
         }
         dispatchEvent(MessageFaultEvent.createEvent(param1));
      }
      
      public function set requestTimeout(param1:int) : void
      {
         _requestTimeout = param1;
      }
      
      protected function initChannelSet(param1:IMessage) : void
      {
         if(_channelSet == null)
         {
            _channelSet = ServerConfig.getChannelSet(destination,param1);
            _channelSet.connect(this);
            if(_credentials != null && _channelSet != null)
            {
               channelSet.setCredentials(_credentials,this);
            }
         }
      }
      
      public function acknowledge(param1:AcknowledgeMessage, param2:IMessage) : void
      {
         if(Log.isInfo())
         {
            _log.info("\'{0}\' {2} acknowledge of \'{1}\'.",id,param2.messageId,_agentType);
         }
         if(param1.headers[AcknowledgeMessage.ERROR_HINT_HEADER])
         {
            delete param1.headers[AcknowledgeMessage.ERROR_HINT_HEADER];
         }
         dispatchEvent(MessageAckEvent.createEvent(param1,param2));
      }
      
      public function set session(param1:Session) : void
      {
         if(_session != param1)
         {
            if(_session != null)
            {
               _session.removeAgent(this);
            }
            _session = param1;
            if(_session != null)
            {
               _session.addAgent(this);
            }
         }
      }
      
      public function disconnect() : void
      {
         channelSet = null;
      }
      
      public function set id(param1:String) : void
      {
         _id = param1;
      }
      
      public function set channelSet(param1:ChannelSet) : void
      {
         if(_channelSet != param1)
         {
            if(_channelSet != null)
            {
               _channelSet.disconnect(this);
               _configured = false;
            }
            else
            {
               _configured = true;
            }
            _channelSet = param1;
            if(_channelSet != null)
            {
               if(_credentials)
               {
                  _channelSet.setCredentials(_credentials,this);
               }
               _channelSet.connect(this);
            }
         }
      }
      
      public function get clientId() : String
      {
         return _clientId;
      }
      
      mx_internal function setClientId(param1:String) : void
      {
         _clientId = param1;
      }
      
      public function setCredentials(param1:String, param2:String) : void
      {
         var _loc3_:String = null;
         var _loc4_:Base64Encoder = null;
         if(param1 == null && param2 == null)
         {
            _credentials = null;
         }
         else
         {
            _loc3_ = param1 + ":" + param2;
            _loc4_ = new Base64Encoder();
            _loc4_.encode(_loc3_);
            _credentials = _loc4_.drain();
         }
         if(channelSet != null)
         {
            channelSet.setCredentials(_credentials,this);
         }
      }
      
      public function channelDisconnectHandler(param1:ChannelEvent) : void
      {
         if(Log.isWarn())
         {
            _log.warn("\'{0}\' {1} channel disconnected.",id,_agentType);
         }
         setConnected(false);
         if(_remoteCredentials != null)
         {
            _sendRemoteCredentials = true;
         }
         dispatchEvent(param1);
      }
      
      protected function setConnected(param1:Boolean) : void
      {
         var _loc2_:PropertyChangeEvent = null;
         if(_connected != param1)
         {
            _loc2_ = PropertyChangeEvent.createUpdateEvent(this,"connected",_connected,param1);
            _connected = param1;
            dispatchEvent(_loc2_);
         }
      }
      
      public function get requestTimeout() : int
      {
         return _requestTimeout;
      }
      
      public function setRemoteCredentials(param1:String, param2:String) : void
      {
         var _loc3_:String = null;
         var _loc4_:Base64Encoder = null;
         if(param1 == null && param2 == null)
         {
            _remoteCredentials = "";
         }
         else
         {
            _loc3_ = param1 + ":" + param2;
            _loc4_ = new Base64Encoder();
            _loc4_.encode(_loc3_);
            _remoteCredentials = _loc4_.drain();
         }
         _sendRemoteCredentials = true;
      }
      
      public function initialized(param1:Object, param2:String) : void
      {
         this.id = param2;
      }
      
      public function get channelSet() : ChannelSet
      {
         return _channelSet;
      }
      
      public function channelConnectHandler(param1:ChannelEvent) : void
      {
         if(Log.isInfo())
         {
            _log.info("\'{0}\' {1} connected.",id,_agentType);
         }
         setConnected(true);
         dispatchEvent(param1);
      }
      
      public function set subtopic(param1:String) : void
      {
         if(_subtopic != param1)
         {
            if(param1 == null)
            {
               _subtopic = "";
            }
            else
            {
               _subtopic = param1;
            }
            dispatchEvent(new Event("subtopicChanged"));
         }
      }
      
      public function channelFaultHandler(param1:ChannelFaultEvent) : void
      {
         if(Log.isWarn())
         {
            _log.warn("\'{0}\' {1} channel faulted with {2} {3}",id,_agentType,param1.faultCode,param1.faultDetail);
         }
         if(!param1.channel.connected)
         {
            setConnected(false);
            if(_remoteCredentials != null)
            {
               _sendRemoteCredentials = true;
            }
         }
         dispatchEvent(param1);
      }
      
      [Bindable(event="subtopicChanged")]
      public function get subtopic() : String
      {
         return _subtopic;
      }
      
      final protected function internalSend(param1:IMessage) : void
      {
         param1.clientId = clientId;
         if(requestTimeout > 0)
         {
            param1.headers[AbstractMessage.REQUEST_TIMEOUT_HEADER] = requestTimeout;
         }
         if(subtopic.length > 0)
         {
            param1.headers[AsyncMessage.SUBTOPIC_HEADER] = subtopic;
         }
         if(_sendRemoteCredentials)
         {
            param1.headers[AbstractMessage.REMOTE_CREDENTIALS_HEADER] = _remoteCredentials;
            _sendRemoteCredentials = false;
         }
         if(channelSet != null)
         {
            channelSet.send(this,param1);
         }
         else
         {
            if(destination.length <= 0)
            {
               throw new InvalidDestinationError(TRANSLATOR.textOf("destinationNotSet"));
            }
            initChannelSet(param1);
            if(channelSet != null)
            {
               channelSet.send(this,param1);
            }
         }
      }
   }
}

